@@ -1,4 +1,8 @@
-"""Pythonic Interface to EBUS Daemon (ebusd)."""
+"""
+Pythonic Interface to EBUS Daemon (ebusd).
+
+:class:`Ebus` represents one connection to a EBUSD instance.
+"""
 try:
     import importlib.metadata as importlib_metadata
 except ModuleNotFoundError:
