@@ -30,6 +30,8 @@ async def _main(args):
     ebus = create_ebus(args)
     await async_load_msgdefs(ebus, args)
     msgdefs = ebus.msgdefs.resolve(args.patterns.split(";"))
+    if args.defaultprio:
+        msgdefs.set_defaultprio(args.defaultprio)
     print(f"Observing {msgdefs.summary()}")
-    async for msg in ebus.async_observe(msgdefs=msgdefs, defaultprio=args.defaultprio, ttl=args.ttl):
+    async for msg in ebus.async_observe(msgdefs=msgdefs, ttl=args.ttl):
         print_msg(msg)
