@@ -24,8 +24,10 @@ async def _main(args):
     ebus = create_ebus(args)
     await async_load_msgdefs(ebus, args)
     msgdefs = ebus.msgdefs.resolve(args.patterns.split(";"), filter_=lambda msgdef: msgdef.read or msgdef.update)
+    if args.defaultprio:
+        msgdefs.set_defaultprio(args.defaultprio)
     print(f"Reading to {msgdefs.summary()}")
     for msgdef in msgdefs:
         if msgdef.read:
-            msg = await ebus.async_read(msgdef, defaultprio=args.defaultprio, ttl=args.ttl)
+            msg = await ebus.async_read(msgdef, ttl=args.ttl)
             print_msg(msg)
