@@ -4,12 +4,12 @@ import collections
 import logging
 
 from .connection import CommandError, Connection
-from .const import AUTO, OK
+from .const import OK
 from .msg import BrokenMsg, filter_msg
 from .msgdecoder import MsgDecoder, UnknownMsgError
 from .msgdefdecoder import decode_msgdef
 from .msgdefs import MsgDefs
-from .util import get_autoprio, repr_
+from .util import repr_
 
 _LOGGER = logging.getLogger(__name__)
 _CMD_FINDMSGDEFS = "find -a -F type,circuit,name,fields"
@@ -67,13 +67,21 @@ class Ebus:
         self.msgdecoder.msgdefs = msgdefs
 
     def __copy__(self):
-        return Ebus(self.host, self.port, timeout=self.timeout, scaninterval=self.scaninterval, msgdefs=self.msgdefs)
+        return Ebus(
+            self.host,
+            self.port,
+            timeout=self.timeout,
+            scaninterval=self.scaninterval,
+            scans=self.scans,
+            msgdefs=self.msgdefs,
+        )
 
     async def async_wait_scancompleted(self):
         """Wait until scan is completed."""
         cnts = []
         while True:
-            cnt = sum([1 async for line in self._async_request(_CMD_FINDMSGDEFS)])
+            await self.connection.async_request(_CMD_FINDMSGDEFS)
+            cnt = sum([1 async for line in self.connection.async_read()])
             cnts.append(cnt)
             if len(cnts) < self.scans or not all(cnt == cnts[-1] for cnt in cnts[-self.scans : -1]):
                 await asyncio.sleep(self.scaninterval)
@@ -93,7 +101,8 @@ class Ebus:
         """Load EBUS Message Definition Codes."""
         _LOGGER.info("load_msgdefcodes()")
         self.msgdefcodes = msgdefcodes = []
-        async for line in self._async_request(_CMD_FINDMSGDEFS):
+        await self.connection.async_request(_CMD_FINDMSGDEFS)
+        async for line in self.connection.async_read():
             line = line.strip()
             if line:
                 try:
@@ -118,25 +127,24 @@ class Ebus:
         for msgdef in sorted(msgdefs, key=lambda msgdef: (msgdef.circuit, msgdef.name)):
             self.msgdefs.add(msgdef)
 
-    async def async_read(self, msgdef, defaultprio=None, ttl=None):
+    async def async_read(self, msgdef, ttl=None):
         """
         Read Message.
 
         Raises:
             ValueError: on decoder error
         """
-        _LOGGER.info(f"read({msgdef!r}, defaultprio={defaultprio!r}, ttl={ttl!r})")
-        setprio = msgdef.setprio or defaultprio
-        if setprio == AUTO:
-            setprio = get_autoprio(msgdef)
+        _LOGGER.info(f"read({msgdef!r}, ttl={ttl!r})")
+        return await self._async_read(msgdef, ttl)
+
+    async def _async_read(self, msgdef, ttl):
         try:
-            lines = tuple(
-                [line async for line in self._async_request("read", msgdef.name, c=msgdef.circuit, p=setprio, m=ttl)]
-            )
+            await self.connection.async_request("read", msgdef.name, c=msgdef.circuit, p=msgdef.setprio, m=ttl)
+            line = await self.connection.async_readresp(check=False)
         except CommandError as exc:
             return BrokenMsg(msgdef, str(exc))
         else:
-            return self.msgdecoder.decode_value(msgdef, lines[0])
+            return self.msgdecoder.decode_value(msgdef, line)
 
     async def async_write(self, msgdef, value, ttl=0):
         """Write Message."""
@@ -148,47 +156,53 @@ class Ebus:
             # Read
             if not msgdef.read:
                 raise ValueError(f"Message is not read-modify-writable '{msgdef}'")
-            readline = tuple(
-                [line async for line in self._async_request("read", msgdef.name, c=msgdef.circuit, m=ttl)]
-            )[0]
-            values = readline.split(";")
+            await self.connection.async_request("read", msgdef.name, c=msgdef.circuit, m=ttl)
+            line = await self.connection.async_readresp(check=False)
+            values = line.split(";")
             # Modify
             for fielddef in msgdef.fields:
                 encvalue = fielddef.type_.encode(value)
-                values[fielddef.idx] = encvalue
+                values[fielddef.idx] = str(encvalue)
         else:
             values = [str(fielddef.type_.encode(value)) for fielddef in msgdef.fields]
         # Write
-        async for _ in self._async_request("write", msgdef.name, ";".join(values), c=msgdef.circuit):
-            pass
+        await self.connection.async_request("write", msgdef.name, ";".join(values), c=msgdef.circuit)
+        resp = await self.connection.async_readresp()
+        if resp != "done":
+            raise CommandError(resp)
 
     async def async_listen(self, msgdefs=None):
         """Listen to EBUSD, decode and yield."""
         _LOGGER.info(f"listen(msgdefs={msgdefs!r})")
-        async for line in self._async_request("listen", infinite=True):
-            if line == "listen started":
-                continue
-            msg = self._decode_line(line)
+        async for msg in self._async_listen(msgdefs):
+            yield msg
+
+    async def _async_listen(self, msgdefs):
+        await self.connection.async_request("listen")
+        resp = await self.connection.async_readresp()
+        if resp != "listen started":
+            raise CommandError(f"Listen could not be started: {resp}")
+        async for line in self.connection.async_read(check=False):
+            msg = self._decode_msg(line)
             msg = filter_msg(msg, msgdefs)
             if msg:
                 yield msg
 
-    async def async_observe(self, msgdefs=None, defaultprio=None, ttl=None):
+    async def async_observe(self, msgdefs=None, ttl=None):
         """
         Observe.
 
-        Read all known messages.
-        Use `find` to get the latest data, if me missed any updates in the
-        meantime and start listening
+        Observe `msgdefs` messages.
         """
-        _LOGGER.info(f"observe(msgdefs={msgdefs!r}, defaultprio={defaultprio!r}, ttl={ttl!r})")
+        _LOGGER.info(f"observe(msgdefs={msgdefs!r}, ttl={ttl!r})")
         msgdefs = msgdefs or self.msgdefs
         data = collections.defaultdict(lambda: None)
 
         # read all
         for msgdef in msgdefs:
             if msgdef.read:
-                msg = await self.async_read(msgdef, defaultprio=defaultprio, ttl=ttl)
+                msg = await self._async_read(msgdef, ttl=ttl)
+                _LOGGER.debug("observe-read: {msg}")
                 msg = filter_msg(msg, msgdefs)
                 if msg:
                     if msg.valid:
@@ -198,24 +212,27 @@ class Ebus:
                 data[msgdef.ident] = None
 
         # find new values (which got updated while we where reading)
-        async for line in self._async_request("find -d"):
-            msg = self._decode_line(line)
+        await self.connection.async_request("find -d")
+        async for line in self.connection.async_read(check=False):
+            msg = self._decode_msg(line)
+            _LOGGER.debug("observe-find: {msg}")
             msg = filter_msg(msg, msgdefs)
             if msg and msg != data[msg.msgdef.ident]:
                 yield msg
                 data[msg.msgdef.ident] = msg
 
         # listen
-        async for msg in self.async_listen(msgdefs=msgdefs):
+        async for msg in self._async_listen(msgdefs):
+            _LOGGER.debug("observe-listen: {msg}")
             yield msg
 
     async def async_get_state(self):
         """Return state string."""
         _LOGGER.info("get_state()")
         try:
-            lines = tuple([line async for line in self._async_request("state")])
-            state = lines[0].split(",")[0]
-            if state == "signal acquired":
+            await self.connection.async_request("state")
+            state = await self.connection.async_readresp()
+            if state.startswith("signal acquired"):
                 return OK
             else:
                 return state
@@ -231,35 +248,25 @@ class Ebus:
         """Return info dict."""
         _LOGGER.info("get_info()")
         info = {}
-        async for line in self._async_request("info"):
-            if line:
-                name, value = line.split(":", 1)
-                info[name.strip()] = value.strip()
+        await self.connection.async_request("info")
+        async for line in self.connection.async_read():
+            name, value = line.split(":", 1)
+            info[name.strip()] = value.strip()
         return info
 
     async def async_cmd(self, cmd, infinite=False, check=False):
         """Send `cmd` to EBUSD and Receive Response."""
-        _LOGGER.info(f"cmd({cmd!r}, infinite={infinite!r})")
+        _LOGGER.info(f"cmd({cmd!r}, infinite={infinite!r}, check={check!r})")
         await self.connection.async_write(cmd)
-        async for line in self.connection.async_readlines(infinite=infinite, check=check):
+        async for line in self.connection.async_read(infinite=infinite, check=check):
             yield line
 
-    async def _async_request(self, cmd, *args, infinite=False, **kwargs):
-        """Assemble request, send and readlines."""
-        parts = [cmd]
-        parts += [f"-{option} {value}" for option, value in kwargs.items() if value is not None]
-        parts += [str(arg) for arg in args]
-        await self.connection.async_write(" ".join(parts))
-        async for line in self.connection.async_readlines(infinite=infinite):
-            yield line
-
-    def _decode_line(self, line):
+    def _decode_msg(self, line):
         if line:
             try:
                 return self.msgdecoder.decode_line(line)
             except UnknownMsgError:
-                return None
+                pass
             except ValueError as exc:
                 _LOGGER.warning(f"Cannot decode message in {line!r}: {exc}")
-        else:
-            return None
+        return None
