@@ -1,9 +1,10 @@
 """Message Defintions."""
 import collections
 import re
-from fnmatch import fnmatchcase
+from fnmatch import fnmatch
 
 from .const import AUTO
+from .util import resolve_prio
 
 
 class MsgDefs:
@@ -67,7 +68,7 @@ class MsgDefs:
         """Find Message Definitions of `circuit` with `name`."""
         msgdefs = MsgDefs()
         for msgdef in self:
-            if fnmatchcase(msgdef.circuit, circuit) and fnmatchcase(msgdef.name, name):
+            if fnmatch(msgdef.circuit, circuit) and fnmatch(msgdef.name, name):
                 msgdefs.add(msgdef)
         return msgdefs
 
@@ -88,7 +89,7 @@ class MsgDefs:
                 if fieldname is None:
                     fields = msgdef.children
                 else:
-                    fields = tuple(fielddef for fielddef in msgdef.children if fnmatchcase(fielddef.name, fieldname))
+                    fields = tuple(fielddef for fielddef in msgdef.children if fnmatch(fielddef.name, fieldname))
                 if not fields:
                     continue
                 if fields == msgdef.children and (setprio is None or not msgdef.read):
@@ -99,6 +100,15 @@ class MsgDefs:
         else:
             raise ValueError(f"Invalid pattern {pattern!r}")
 
+    def set_defaultprio(self, defaultprio):
+        """Set Default Priorities."""
+        msgdefs = self._msgdefs
+        for circuitmsgdefs in msgdefs.values():
+            for msgdefs in circuitmsgdefs.values():
+                for idx, msgdef in enumerate(msgdefs):
+                    if msgdef.setprio is None:
+                        msgdefs[idx] = msgdef.replace(setprio=resolve_prio(msgdef, msgdef.setprio or defaultprio))
+
     def summary(self):
         """Summary."""
         total = len(self)
