@@ -1,5 +1,7 @@
 """Utilities."""
 
+from .const import AUTO
+
 
 def repr_(obj, args=None, kwargs=None):
     """
@@ -18,16 +20,25 @@ def repr_(obj, args=None, kwargs=None):
     return "%s(%s)" % (classname, ", ".join(args))
 
 
-def get_autoprio(msgdef):
+HIGHPRIO_KEYWORDS = ("temp", "status")
+LOWPRIO_KEYWORDS = ("error", "min", "max", "param", "offset", "name", "hours", "energy", "yield")
+
+
+def resolve_prio(msgdef, setprio):
     """Calculate best priority."""
     if msgdef.read:
-        # Prio is only available on read messages.
-        if msgdef.update or msgdef.write:
-            # we just want to ensure that we do not miss any value
-            # change, so just poll every second iteration.
-            return 2
-        else:
-            return 1
+        if setprio == AUTO:
+            # Prio is only available on read messages.
+            namelower = msgdef.name.lower()
+            if msgdef.update or msgdef.write or any(keyword in namelower for keyword in LOWPRIO_KEYWORDS):
+                # we just want to ensure that we do not miss any value
+                setprio = 3
+            elif any(keyword in namelower for keyword in HIGHPRIO_KEYWORDS):
+                # we just want to ensure that we do not miss any value
+                setprio = 1
+            else:
+                setprio = 2
+        return setprio
     else:
         return None
 
